#!/usr/bin/python2.7
# coding=UTF-8
import sys
import re
import gtk

def valid_username(u, match=re.compile(r'^[a-z][-_a-z0-9]*[a-z0-9]$').match):
    """Usernames must start with a letter, end with a letter or a digit, and may also contain dashes and underscores."""
    return bool(match(u))

def valid_password(p, match=re.compile(r'^[^\x00-\x1F\x7F]+$').match):
    """Passwords must not be empty, and must not contain ASCII control characters."""
    return bool(match(p))

# ❀ ☢

class Authenticate:

    def __init__(self, title="Authenticate", 
                       text="Please identify yourself",
                       label="University of Helsinki",
                       passchar="☢",
                       width=320, height=180):
        self.Username = ''
        self.Password = ''

        screen_width = gtk.gdk.screen_width()
        screen_height = gtk.gdk.screen_height()

        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.connect("delete_event", self.Cancel)
        self.window.set_title(title)
        self.window.move((screen_width - width)/2, (screen_height - height)/2)
        self.window.resize(width, height)

        self.header = gtk.Label(text)
        self.header.set_justify(gtk.JUSTIFY_CENTER)
        self.header.set_alignment(0.5, 0.5)

        self.userlabel = gtk.Label("Username:")
        self.userlabel.set_justify(gtk.JUSTIFY_RIGHT)
        self.userlabel.set_alignment(1.0, 0.5)

        self.userentry = gtk.Entry(0)
        self.userentry.connect("activate", self.user_enter)
        self.userentry.connect("changed", self.changed)

        self.passlabel = gtk.Label("Password:")
        self.passlabel.set_justify(gtk.JUSTIFY_RIGHT)
        self.passlabel.set_alignment(1.0, 0.5)

        self.passentry = gtk.Entry(0)
        self.passentry.set_invisible_char(passchar)
        self.passentry.set_visibility(False)
        self.passentry.connect("activate", self.pass_enter)
        self.passentry.connect("changed", self.changed)

        self.loginbutton = gtk.Button("   Login   ")
        self.loginbutton.connect("clicked", self.Login)
        self.loginbutton.set_sensitive(False)

        self.cancelbutton = gtk.Button("   Cancel   ")
        self.cancelbutton.connect("clicked", self.Cancel)
	self.cancelbutton.set_sensitive(True)

        table = gtk.Table(rows=2, columns=2, homogeneous=False)
        table.attach(self.userlabel, 0, 1, 0, 1, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND, xpadding=0, ypadding=2)
        table.attach(self.userentry, 1, 2, 0, 1, xoptions=gtk.EXPAND,          yoptions=gtk.EXPAND, xpadding=0, ypadding=2)
        table.attach(self.passlabel, 0, 1, 1, 2, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND, xpadding=0, ypadding=2)
        table.attach(self.passentry, 1, 2, 1, 2, xoptions=gtk.EXPAND,          yoptions=gtk.EXPAND, xpadding=0, ypadding=2)

        frame = gtk.Frame(label)
        frame.add(table)

        hbox = gtk.HBox(homogeneous=True, spacing=2)
        hbox.add_with_properties(self.cancelbutton, "fill", False, "expand", True)
        hbox.add_with_properties(self.loginbutton, "fill", False, "expand", True)
        hbox.set_focus_chain([self.loginbutton, self.cancelbutton])

        empty = gtk.HBox(spacing=4)

        vbox = gtk.VBox(homogeneous=False, spacing=2)
        vbox.add_with_properties(self.header, "fill", True, "expand", True)
        vbox.add_with_properties(frame, "fill", True, "expand", False)
        vbox.add_with_properties(hbox, "fill", False, "expand", False)
        vbox.add_with_properties(empty, "fill", True, "expand", True)

        self.window.add(vbox)

        self.window.show_all()

    def changed(self, *args):
        self.loginbutton.set_sensitive(valid_username(self.userentry.get_text()) and
                                       valid_password(self.passentry.get_text()))

    def user_enter(self, *args):
        self.passentry.grab_focus()

    def pass_enter(self, *args):
        self.Login()
 
    def Cancel(self, *args):
        self.Username = ''
        self.Password = ''
        gtk.main_quit()
	return False

    def Login(self, *args):
        u = self.userentry.get_text()
        p = self.passentry.get_text()
        if valid_username(u) and valid_password(p):
            self.Username = u
            self.Password = p
            gtk.main_quit()

if __name__ == '__main__':

    window = Authenticate()
    gtk.main()

    sys.stdout.write(str(window.Username))
    sys.stdout.write("\n")
    sys.stdout.write(str(window.Password))
    sys.stdout.write("\n")
